.module levels
.include "man/levels_public.s"
.include "cpctelera.h.s"

.globl cpct_memset_asm
.globl cpct_memcpy_asm
.globl cpct_getRandom_xsp40_u8_asm


;;
;; LEVELS ARRAY. This is where the level data will be stored
;;
levels_array::
level_1:
    .db #0, #5, #0, #0  ;; max_time 
    .db #1  ;; pattern
    .db #25  ;; spawn_time
    .db #0   ;; wizard_spawn

level_2:
    .db #0, #5, #0, #0  ;; max_time 
    .db #2  ;; pattern
    .db #25  ;; spawn_time
    .db #0   ;; wizard_spawn

level_3:
    .db #0, #3, #0, #0  ;; max_time 
    .db #3  ;; pattern
    .db #25  ;; spawn_time
    .db #1   ;; wizard_spawn

level_4:
    .db #0, #6, #0, #0  ;; max_time 
    .db #4  ;; pattern
    .db #25  ;; spawn_time
    .db #1   ;; wizard_spawn

level_5:
    .db #0, #7, #5, #0  ;; max_time 
    .db #5  ;; pattern
    .db #25  ;; spawn_time
    .db #1   ;; wizard_spawn

;;
;; CURRENT LEVEL. This is where the number of the current level is stored
;;
current_level::
   .db 0
;;

;;
;; PATTERN STEP. This is where the step of the pattern of the current level is stored
;;
pattern_step::
   .db 0
;;

;;
;; GET CURRENT LEVEL. This returns the current level's adress in the array
;; OUTPUTS
;; - IX: Address for the current level
;; DESTROYED REGISTERS: [A, BC, DE, HL]
;;
man_levels_get_current_level::
    ld a, (current_level)
    ld hl, #levels_array

    dec a
    or a
    jr z, _end_get_current_level

    _loop_get_current_level:
        ld b, #0
        ld c, #BYTES_PER_LEVEL

        add hl, bc
        
        dec a
        or a

        jr nz, _loop_get_current_level
    
    _end_get_current_level:
        ex de, hl
        ld__ixh_d
        ld__ixl_e

    ret
;;

;;
;; LEVELS INIT function. This sets the "current_level" and "pattern_step" value to 0
;; DESTROYED REGISTERS: [A = 0]
;;
man_levels_init::
    xor a ;; a = 0
    ld (current_level), a
    ld (pattern_step), a

   ret
;;

;;
;; LEVELS RESET PATTERN STEP function. This resets the "pattern_step" value to 0
;; DESTROYED REGISTERS: [A = 0]
;;
man_levels_reset_pattern_step::
    xor a ;; a = 0
    ld (pattern_step), a

   ret
;;

;;
;; LEVELS INCREASE PATTERN STEP function. This increases the "pattern_step" value  by 1
;; DESTROYED REGISTERS: [A]
;;
man_levels_increase_pattern_step::
    ld a, (pattern_step)
    inc a
    ld (pattern_step), a

   ret
;;

;;
;; LEVELS UPDATE PATTERN STEP function. This updates the "pattern_step" value
;; INPUTS
;; - IX: Address for the current level
;; DESTROYED REGISTERS: [A]
;;
man_levels_update_pattern_step::
    ld a, _pattern_l(ix)
    cp #1
    jr z, _increase_pattern_1_4
    cp #2
    jr z, _increase_pattern_2
    cp #3
    jr z, _increase_pattern_3_5
    cp #4
    jr z, _increase_pattern_1_4
    cp #5
    jr z, _increase_pattern_3_5

    _increase_pattern_1_4:
        ret

    _increase_pattern_2:
        call man_levels_increase_pattern_step
        cp #3
        ret nz

        call man_levels_reset_pattern_step
        ret
         

    _increase_pattern_3_5:
        call man_levels_increase_pattern_step
        cp #2
        ret nz

        call man_levels_reset_pattern_step
        ret
;;