.module render
.include "cpctelera.h.s"
.include "man/entities_public.s"
.include "man/levels_public.s"


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;; GAME SYMBOLS DEFINED ELSEWHERE
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 .globl _g_palette
 .globl _sp_background_0
 .globl _sp_background_1
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


.globl cpct_setVideoMode_asm
.globl cpct_setPalette_asm
.globl cpct_getScreenPtr_asm
.globl cpct_drawSprite_asm
.globl cpct_px2byteM0_asm

.globl man_entities_forall
.globl man_entities_forall_matching

timer_string: .asciz "TIMER";
timer_x_position = 2
timer_y_position = 5

score_string: .asciz "SCORE";
score_x_position = 30
score_y_position = 5
final_score_string: .asciz "FINAL SCORE:";
final_score_x_position = 5
final_score_y_position = 0

level_string: .asciz "LVL:";
level_x_position = timer_x_position
level_y_position = 25

;;
;; Live.
;;
hud_bar::
   .db #58, #30, #20, #12, #4, #5
;;
hb_x=0
hb_y=1
hb_w=2
hb_h=3
hb_col_bg=4
hb_col_fg=5


;;
;; RENDER INIT function. This sets the video mode, palette and background
;;
sys_render_init::
   ld   c, #0
   call cpct_setVideoMode_asm
   ld   hl, #_g_palette
   ld   de, #16
   call cpct_setPalette_asm
   
   cpctm_setBorder_asm HW_BLACK

   call sys_render_set_red_char_background
	
   ret
;;

sys_render_set_red_char_background::

    ld   h, #0
    ld   l, #15
    call cpct_setDrawCharM0_asm
	
   ret
;;

sys_render_set_black_char_background::

    ld   h, #3
    ld   l, #15
    call cpct_setDrawCharM0_asm
	
   ret
;;


sys_render_update_one_entity::
        ;; Check entity type not e_type_dead and before drawing
        ld a, _type_e (ix)
        and #e_type_dead
        ret nz

        ;; Calculate new location
        ld de, #CPCT_VMEM_START_ASM
        ld c, _x_e (ix)
        ld b, _y_e (ix)
        call cpct_getScreenPtr_asm

        ;; Draw Sprite
        ex de, hl                       ;;DE = pvmem(where to draw the)
        ld l, _sprite_e (ix)  
        ld h, _sprite_e +1 (ix)         ;; Address where sprite data start
        ld c, _w_e (ix)                 ;; Sprite width
        ld b, _h_e (ix)                 ;; Sprite heigth
        call cpct_drawSprite_asm
    ret
;;


;;
;; RENDER BOX BACKGROUND COLOR function. 
;; INPUTS
;; - IX: Entity direction (dead) to be cleared
sys_render_background_color_box::
    ;; Calculate new location
    ld de, #CPCT_VMEM_START_ASM
    ld c, _x_e(ix) 
    ld b, _y_e(ix) 
    call cpct_getScreenPtr_asm

    ;; Draw Sprite
    ex de, hl                       ;;DE = pvmem(where to draw the)


    ;; Draw Sprite
    ld a, #0
    ld c, _w_e(ix)                 ;; Box width
    ld b, _h_e(ix)                 ;; Box heigth
    call cpct_drawSolidBox_asm

    ret
;;

;;
;; RENDER HEALTH BAR COLOR function. 
;; INPUTS
;; - IX: Entity direction (dead) to be cleared
sys_render_health_bar::
    ld ix, #hud_bar


    ;;BARRA VERDE
    ;; Calculate new location
    ld de, #CPCT_VMEM_START_ASM
    ld c, hb_x(ix) 
    ld b, hb_y(ix) 
    call cpct_getScreenPtr_asm

    ;; Draw Sprite
    ex de, hl                       ;;DE = pvmem(where to draw the)

    ld a, (player_health)
    add a, a 
    add a, a
    ld c, a                 ;; Box width
    ld b, hb_h(ix)          ;; Box heigth
    push bc

    push de
    ld h, hb_col_fg(ix)
    ld l, hb_col_fg(ix)

    call cpct_px2byteM0_asm
    pop de
    pop bc
    push bc

    ;; Draw Sprite
    call cpct_drawSolidBox_asm


    ;; BARRA ROJA
    pop bc
    push bc
    ;; Calculate new location
    ld de, #CPCT_VMEM_START_ASM
    ld a, c
    add a, hb_x(ix)
    ld c, a          ;; ROJO X = X de la barra + player_health * 4
    ld b, hb_y(ix) 
    call cpct_getScreenPtr_asm

    ;; Draw Sprite
    ex de, hl                       ;;DE = pvmem(where to draw the)


    pop bc
    ld a, hb_w(ix)            
    sub a, c
    ld c, a                   ;; ROJO W = 20 - player_health *4
    ld b, hb_h(ix)                 ;; Box heigth

    push bc
    push de
    ld h, hb_col_bg(ix)
    ld l, hb_col_bg(ix)

    call cpct_px2byteM0_asm
    pop de
    pop bc
    call cpct_drawSolidBox_asm





    ret
;;



;; RENDER END DOOR function. This renders the four bytes of the timer on the screen
;; DESTROYED REGISTERS: [BC, DE, HL]
;;
sys_render_end_door:
    ;; Calculate location for end door
    ld de, #CPCT_VMEM_START_ASM
    ld c, #70
    ld b, #132
    call cpct_getScreenPtr_asm

    ;; Draw Sprite
    ex de, hl                       ;;DE = pvmem(where to draw the)


    ;; Draw Sprite (W=10, H=30)
    ld hl, #_sp_end_door
    ld c, #8                 ;; Sprite width
    ld b, #40                 ;; Sprite heigth
    call cpct_drawSprite_asm

    ret
;;

;; RENDER LIVES function.
;; DESTROYED REGISTERS: [BC, DE, HL]
;;
sys_render_lives:
    ;; Calculate location for end door
    ld de, #CPCT_VMEM_START_ASM
    ld c, #56
    ld b, #0
    call cpct_getScreenPtr_asm

    ;; Draw Sprite
    ex de, hl                       ;;DE = pvmem(where to draw the)


    ;; Draw Sprite (W=10, H=30)
    ld a, (lives)
    cp a, #3
    jr nz, vidas2

    ld hl, #_sp_live3
    vidas2:
    cp a, #2
    jr nz, vidas1
    ld hl, #_sp_live2
    vidas1:
    cp a, #1
    jr nz, vidas0
    ld hl, #_sp_live1

    vidas0:
    or a
    ret z

    ld c, #24                 ;; Sprite width
    ld b, #24                 ;; Sprite heigth
    call cpct_drawSprite_asm

    ret
;;


sys_render_background::
    ;; Calculate new location
    ld de, #CPCT_VMEM_START_ASM
    ld c, #0
    ld b, #173
    call cpct_getScreenPtr_asm

    ;; Draw Sprite
    ex de, hl                       ;;DE = pvmem(where to draw the)


    ;; Draw Sprite
    ld hl, #_sp_background_0
    ld c, #40                 ;; Sprite width
    ld b, #27                 ;; Sprite heigth
    call cpct_drawSprite_asm

    ;; Calculate new location
    ld de, #CPCT_VMEM_START_ASM
    ld c, #40
    ld b, #173
    call cpct_getScreenPtr_asm

    ex de,hl 
    ;; Draw Sprite
    ld hl, #_sp_background_1
    ld c, #40                 ;; Sprite width
    ld b, #27                 ;; Sprite heigth
    call cpct_drawSprite_asm

    ret
;;

;; RENDER UPDATE function. This renders the pixels of the entities (stars) on the screen
sys_render_update::
    call sys_render_end_door

    ld  hl, #sys_render_update_one_entity
    ld b, #e_type_render
    call man_entities_forall_matching


    call sys_render_health_bar
    call sys_render_lives
  
    call sys_render_timer
    call sys_render_score
    call sys_render_level_index

    ret
;;

;;
;; RENDER TIMER function. This renders the four bytes of the timer on the screen
;; DESTROYED REGISTERS: [A, B, DE, HL, IX, IY]
;;
sys_render_timer::
    ld  ix, #timer

    ;; First character of timer
    ld   a, 0(ix)

    ;; Convert the byte value to ASCII code (+48)
    add  a, #48

    cpctm_push ix, af

    ;; Draw string on screen
    ld  de, #CPCT_VMEM_START_ASM
    ld   c, #timer_x_position
    ld   b, #timer_y_position
    call cpct_getScreenPtr_asm

    ld   iy, #timer_string          ;; IY = Pointer to the string 

    call cpct_drawStringM0_asm      ;; Draw the string

    pop     af
    push    af

    ;; Draw character on screen
    ld  de, #CPCT_VMEM_START_ASM
    ld   c, #timer_x_position
    ld   a, #0                       ;; Load into A position of the current digit
    add  a, a
    add  a, a
    add  a, c                       ;; Digit x position = timer_x_position + 4 * position of the current digit (2 px per char)
    ld   c, a
    ld   b, #timer_y_position+10
    call cpct_getScreenPtr_asm

    pop     af
    push    af


    ld   e, a
    call cpct_drawCharM0_asm

    cpctm_pop af, ix



    ;; Second character of timer
    ld   a, 1(ix)

    ;; Convert the byte value to ASCII code (+48)
    add  a, #48

    cpctm_push ix, af

    ;; Draw character on screen
    ld  de, #CPCT_VMEM_START_ASM
    ld   c, #timer_x_position
    ld   a, #1                       ;; Load into A position of the current digit
    add  a, a
    add  a, a
    add  a, c                       ;; Digit x position = timer_x_position + 4 * position of the current digit (2 px per char)
    ld   c, a
    ld   b, #timer_y_position+10
    call cpct_getScreenPtr_asm

    pop af
    push af


    ld   e, a
    call cpct_drawCharM0_asm

    cpctm_pop af, ix



    ;; Third character of timer
    ld   a, 2(ix)

    ;; Convert the byte value to ASCII code (+48)
    add  a, #48

    cpctm_push ix, af

    ;; Draw character on screen
    ld  de, #CPCT_VMEM_START_ASM
    ld   c, #timer_x_position
    ld   a, #2                       ;; Load into A position of the current digit
    add  a, a
    add  a, a
    add  a, c                       ;; Digit x position = timer_x_position + 4 * position of the current digit (2 px per char)
    ld   c, a
    ld   b, #timer_y_position+10
    call cpct_getScreenPtr_asm

    pop af
    push af


    ld   e, a
    call cpct_drawCharM0_asm

    cpctm_pop af, ix



    ;; Fourth (and final) character of timer
    ld   a, 3(ix)

    ;; Convert the byte value to ASCII code (+48)
    add  a, #48

    cpctm_push ix, af

    ;; Draw character on screen
    ld  de, #CPCT_VMEM_START_ASM
    ld   c, #timer_x_position
    ld   a, #3                      ;; Load into A position of the current digit
    add  a, a
    add  a, a
    add  a, c                       ;; Digit x position = timer_x_position + 4 * position of the current digit (2 px per char)
    ld   c, a
    ld   b, #timer_y_position+10
    call cpct_getScreenPtr_asm

    pop af
    push af


    ld   e, a
    call cpct_drawCharM0_asm

    cpctm_pop af, ix

    ret
;;

;;
;; RENDER SCORE function. This renders the four bytes of the score on the screen
;; DESTROYED REGISTERS: [A, B, DE, HL, IX, IY]
;;
sys_render_score::
    ld  ix, #score

    ;; First character of score
    ld   a, 0(ix)

    ;; Convert the byte value to ASCII code (+48)
    add  a, #48

    cpctm_push ix, af

    ;; Draw string on screen
    ld  de, #CPCT_VMEM_START_ASM
    ld   c, #score_x_position
    ld   b, #score_y_position
    call cpct_getScreenPtr_asm

    ld   iy, #score_string          ;; IY = Pointer to the string 

    call cpct_drawStringM0_asm      ;; Draw the string

    pop     af
    push    af

    ;; Draw character on screen
    ld  de, #CPCT_VMEM_START_ASM
    ld   c, #score_x_position
    ld   a, #0                       ;; Load into A position of the current digit
    add  a, a
    add  a, a
    add  a, c                       ;; Digit x position = score_x_position + 4 * position of the current digit (2 px per char)
    ld   c, a
    ld   b, #score_y_position+10
    call cpct_getScreenPtr_asm

    pop     af
    push    af


    ld   e, a
    call cpct_drawCharM0_asm

    cpctm_pop af, ix



    ;; Second character of score
    ld   a, 1(ix)

    ;; Convert the byte value to ASCII code (+48)
    add  a, #48

    cpctm_push ix, af

    ;; Draw character on screen
    ld  de, #CPCT_VMEM_START_ASM
    ld   c, #score_x_position
    ld   a, #1                       ;; Load into A position of the current digit
    add  a, a
    add  a, a
    add  a, c                       ;; Digit x position = score_x_position + 4 * position of the current digit (2 px per char)
    ld   c, a
    ld   b, #score_y_position+10
    call cpct_getScreenPtr_asm

    pop af
    push af


    ld   e, a
    call cpct_drawCharM0_asm

    cpctm_pop af, ix



    ;; Third character of score
    ld   a, 2(ix)

    ;; Convert the byte value to ASCII code (+48)
    add  a, #48

    cpctm_push ix, af

    ;; Draw character on screen
    ld  de, #CPCT_VMEM_START_ASM
    ld   c, #score_x_position
    ld   a, #2                       ;; Load into A position of the current digit
    add  a, a
    add  a, a
    add  a, c                       ;; Digit x position = score_x_position + 4 * position of the current digit (2 px per char)
    ld   c, a
    ld   b, #score_y_position+10
    call cpct_getScreenPtr_asm

    pop af
    push af


    ld   e, a
    call cpct_drawCharM0_asm

    cpctm_pop af, ix



    ;; Fourth (and final) character of score
    ld   a, 3(ix)

    ;; Convert the byte value to ASCII code (+48)
    add  a, #48

    cpctm_push ix, af

    ;; Draw character on screen
    ld  de, #CPCT_VMEM_START_ASM
    ld   c, #score_x_position
    ld   a, #3                      ;; Load into A position of the current digit
    add  a, a
    add  a, a
    add  a, c                       ;; Digit x position = score_x_position + 4 * position of the current digit (2 px per char)
    ld   c, a
    ld   b, #score_y_position+10
    call cpct_getScreenPtr_asm

    pop af
    push af


    ld   e, a
    call cpct_drawCharM0_asm

    cpctm_pop af, ix

    ret
;;

;;
;; RENDER FINAL SCORE function. This renders the four bytes of the score on the end screen
;; DESTROYED REGISTERS: [A, B, DE, HL, IX, IY]
;;
sys_render_final_score::
    ld  ix, #score

    ;; First character of score
    ld   a, 0(ix)

    ;; Convert the byte value to ASCII code (+48)
    add  a, #48

    cpctm_push ix, af

    ;; Draw string on screen
    ld  de, #CPCT_VMEM_START_ASM
    ld   c, #final_score_x_position
    ld   b, #final_score_y_position
    call cpct_getScreenPtr_asm

    ld   iy, #final_score_string    ;; IY = Pointer to the string 

    call cpct_drawStringM0_asm      ;; Draw the string

    pop     af
    push    af

    ;; Draw character on screen
    ld  de, #CPCT_VMEM_START_ASM
    ld   c, #final_score_x_position + 52
    ld   a, #0                       ;; Load into A position of the current digit
    add  a, a
    add  a, a
    add  a, c                       ;; Digit x position = final_score_x_position + 52 + 4 * position of the current digit (2 px per char)
    ld   c, a
    ld   b, #final_score_y_position
    call cpct_getScreenPtr_asm

    pop     af
    push    af


    ld   e, a
    call cpct_drawCharM0_asm

    cpctm_pop af, ix



    ;; Second character of score
    ld   a, 1(ix)

    ;; Convert the byte value to ASCII code (+48)
    add  a, #48

    cpctm_push ix, af

    ;; Draw character on screen
    ld  de, #CPCT_VMEM_START_ASM
    ld   c, #final_score_x_position + 52
    ld   a, #1                       ;; Load into A position of the current digit
    add  a, a
    add  a, a
    add  a, c                       ;; Digit x position = final_score_x_position + 52 + 4 * position of the current digit (2 px per char)
    ld   c, a
    ld   b, #final_score_y_position
    call cpct_getScreenPtr_asm

    pop af
    push af


    ld   e, a
    call cpct_drawCharM0_asm

    cpctm_pop af, ix



    ;; Third character of score
    ld   a, 2(ix)

    ;; Convert the byte value to ASCII code (+48)
    add  a, #48

    cpctm_push ix, af

    ;; Draw character on screen
    ld  de, #CPCT_VMEM_START_ASM
    ld   c, #final_score_x_position + 52
    ld   a, #2                       ;; Load into A position of the current digit
    add  a, a
    add  a, a
    add  a, c                       ;; Digit x position = final_score_x_position + 52 + 4 * position of the current digit (2 px per char)
    ld   c, a
    ld   b, #final_score_y_position
    call cpct_getScreenPtr_asm

    pop af
    push af


    ld   e, a
    call cpct_drawCharM0_asm

    cpctm_pop af, ix



    ;; Fourth (and final) character of score
    ld   a, 3(ix)

    ;; Convert the byte value to ASCII code (+48)
    add  a, #48

    cpctm_push ix, af

    ;; Draw character on screen
    ld  de, #CPCT_VMEM_START_ASM
    ld   c, #final_score_x_position + 52
    ld   a, #3                      ;; Load into A position of the current digit
    add  a, a
    add  a, a
    add  a, c                       ;; Digit x position = final_score_x_position + 52 + 4 * position of the current digit (2 px per char)
    ld   c, a
    ld   b, #final_score_y_position
    call cpct_getScreenPtr_asm

    pop af
    push af


    ld   e, a
    call cpct_drawCharM0_asm

    cpctm_pop af, ix

    ret
;;

;;
;; RENDER LEVEL INDEX function. This renders the four bytes of the timer on the screen
;; DESTROYED REGISTERS: [A, B, DE, HL, IX, IY]
;;
sys_render_level_index:: 
    ;; Draw string on screen
    ld  de, #CPCT_VMEM_START_ASM
    ld   c, #level_x_position
    ld   b, #level_y_position
    call cpct_getScreenPtr_asm

    ld   iy, #level_string          ;; IY = Pointer to the string 

    call cpct_drawStringM0_asm      ;; Draw the string
    
    ;; Level index
    ld   a, (current_level)

    ;; Convert the byte value to ASCII code (+48)
    add  a, #48

    push    af

    ;; Draw character on screen
    ld  de, #CPCT_VMEM_START_ASM
    ld   c, #level_x_position+16
    ld   b, #level_y_position
    call cpct_getScreenPtr_asm

    pop     af

    ld   e, a
    call cpct_drawCharM0_asm

    ;; Slash
    ld   a, #'/'

    push af

    ;; Draw character on screen
    ld  de, #CPCT_VMEM_START_ASM
    ld   c, #level_x_position+20
    ld   b, #level_y_position
    call cpct_getScreenPtr_asm

    pop af

    ld   e, a
    call cpct_drawCharM0_asm

    ;; Total number of levels
    ld   a, #NUM_LEVELS

    ;; Convert the byte value to ASCII code (+48)
    add  a, #48

    push af

    ;; Draw character on screen
    ld  de, #CPCT_VMEM_START_ASM
    ld   c, #level_x_position+24
    ld   b, #level_y_position
    call cpct_getScreenPtr_asm

    pop af

    ld   e, a
    call cpct_drawCharM0_asm

    ret
;;