.module time
.include "man/levels_public.s"

.globl man_game_level_reset

;;
;; TIMER. This is where the values for the timer is stored
;; Four bytes, 0-9 values for each
;;
timer::
.db #1, #0, #0, #0

;;
;; CREATE ENEMY TIMER. This is where the timer for enemy generation is stored
;;
create_enemy_timer::
   .db #0
;;

;;
;; TIME INIT function. This sets the timer to 0
;; DESTROYED REGISTERS: [A, IX]
;;
sys_time_init::
    ld  ix, #timer
    ld   0(ix), #1
    ld   1(ix), #1
    ld   2(ix), #1
    ld   3(ix), #1

    xor a
    ld (create_enemy_timer), a

    ret
;;

;;
;; TIME LEVEL INIT function. This sets the timer to the "_max_time_l" of the level
;; INPUTS
;; - IY: Adress of the current level
;; DESTROYED REGISTERS: [A, IX]
;;
sys_time_level_init::
    ld  ix, #timer
    ld   a, _max_time_l+0(iy)
    ld   0(ix), a
    ld   a, _max_time_l+1(iy)
    ld   1(ix), a
    ld   a, _max_time_l+2(iy)
    ld   2(ix), a
    ld   a, _max_time_l+3(iy)
    ld   3(ix), a

    ld   a, _spawn_time_l(iy)
    ld   (create_enemy_timer), a

    ret
;;

;;
;; TIME UPDATE function. This updates the current timer value, substracting by one
;; INPUTS
;; - IY: Adress of the current level
;; DESTROYED REGISTERS: [A, IX, IY]
;;
sys_time_update::
    ld   a, (create_enemy_timer)
    dec  a
    jp   p, _no_create_enemy_timer_overflow

    ld   a, _spawn_time_l(iy)

    _no_create_enemy_timer_overflow:
    ld   (create_enemy_timer), a
    ld  ix, #timer
    
    _update_timer_3:
        ;; Check if the digit value is 0
        ld a, 3(ix)
        or a
        jr nz, _timer_3_value_not_0


        ;; If the digit position isn't 0, set value to 9, update value for previous digit and return
        ld 3(ix), #9

        _update_timer_2:
            ;; Check if the digit value is 0
            ld a, 2(ix)
            or a
            jr nz, _timer_2_value_not_0


            ;; If the digit position isn't 0, set value to 9, update value for previous digit and return
            ld 2(ix), #9

            _update_timer_1:
                ;; Check if the digit value is 0
                ld a, 1(ix)
                or a
                jr nz, _timer_1_value_not_0


                ;; If the digit position isn't 0, set value to 9, update value for previous digit and return
                ld 1(ix), #9

                _update_timer_0:
                    ;; Check if the digit value is 0
                    ld a, 0(ix)
                    or a
                    jr nz, _timer_0_value_not_0

                    ;; If the digit position is 0 and value is 0, call "sys_time_run_out" and return
                    call sys_time_run_out

                    ret

                    ;; If the digit value isn't 0, substract 1 and return
                    _timer_0_value_not_0:
                        dec 0(ix)
                    ret
                ret

                ;; If the digit value isn't 0, substract 1 and return
                _timer_1_value_not_0:
                    dec 1(ix)
                ret

            ;; If the digit value isn't 0, substract 1 and return
            _timer_2_value_not_0:
                dec 2(ix)
            ret

        ;; If the digit value isn't 0, substract 1 and return
        _timer_3_value_not_0:
            dec 3(ix)
        ret
;;

;;
;; TIME RUN OUT function. This is called when the timer runs out for the level
;; UNDER CONSTRUCTION: Requires levels to finish
;; DESTROYED REGISTERS (provisional): [IX]
;;
sys_time_run_out:
    call man_game_level_reset

    ret
;;